package com.uas.erp.manage.client.service;

import com.uas.ops.diff.domain.Manifest;
import com.uas.erp.manage.client.domain.MavenMetadata;
import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.entity.ArtifactItem;
import com.uas.erp.manage.client.entity.Project;
import com.uas.erp.manage.client.repository.ArtifactItemRepository;
import com.uas.erp.manage.client.repository.ArtifactRepository;
import com.uas.erp.manage.client.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/6/19.
 */
@Service
public class ArtifactService {

    @Autowired
    private ArtifactRepository artifactRepository;

    @Autowired
    private ArtifactItemRepository artifactItemRepository;

    @Autowired
    private LocalRepositoryService localRepository;

    @Autowired
    private RemoteRepositoryService remoteRepository;

    @Autowired
    private ProjectService projectService;

    private Map<Long, String> artifactUpdateTasks = new HashMap<Long, String>();

    public Artifact getArtifact(long id) {
        return artifactRepository.findOne(id);
    }

    public Artifact getArtifact(String groupId, String artifactId) {
        return artifactRepository.findByGroupIdAndArtifactId(groupId, artifactId);
    }

    public ArtifactItem getArtifactItem(long artifactId, String version) {
        return artifactItemRepository.findByArtifactIdAndVersion(artifactId, version);
    }

    public ArtifactItem getArtifactItem(long artifactItemId) {
        return artifactItemRepository.findOne(artifactItemId);
    }

    public ArtifactItem getArtifactItem(String groupId, String artifactId, String version) {
        Artifact artifact = getArtifact(groupId, artifactId);
        if (null != artifact) {
            return getArtifactItem(artifact.getId(), version);
        }
        return null;
    }

    public List<ArtifactItem> getArtifactItems(long id) {
        Artifact artifact = getArtifact(id);
        if (null != artifact) {
            return artifactItemRepository.findByArtifactIdOrderByVersionDesc(artifact.getId());
        }
        return null;
    }

    public List<ArtifactItem> getArtifactItems(String groupId, String artifactId) {
        Artifact artifact = getArtifact(groupId, artifactId);
        if (null != artifact) {
            return artifactItemRepository.findByArtifactIdOrderByVersionDesc(artifact.getId());
        }
        return null;
    }

    public List<Artifact> getArtifacts() {
        return artifactRepository.findAll();
    }

    public void safetySave(Artifact artifact) {
        Artifact oldArtifact = getArtifact(artifact.getGroupId(), artifact.getArtifactId());
        if (null != oldArtifact) {
            oldArtifact.setDescription(artifact.getDescription());
            oldArtifact.setAutoUpdate(artifact.isAutoUpdate());
        } else {
            oldArtifact = artifact;
        }
        artifactRepository.save(oldArtifact);
    }

    public void save(Artifact artifact) {
        artifactRepository.save(artifact);
    }

    public void save(List<Artifact> artifacts) {
        artifactRepository.save(artifacts);
    }

    @Transactional
    public void delete(Artifact artifact) {
        List<Project> projects = projectService.findByArtifactId(artifact.getId());
        Assert.isTrue(CollectionUtils.isEmpty(projects), "有项目正在使用该软件，无法删除");
        Assert.isNull(getUpdateTask(artifact.getId()), "软件正在更新，请稍后操作");
        artifactItemRepository.deleteByArtifactId(artifact.getId());
        artifactRepository.delete(artifact);
    }

    public void deleteAll() {
        artifactRepository.deleteAll();
    }
    /**
     * 更新到远程仓库的最新版
     * @param artifact
     * @throws Exception
     */
    public ArtifactItem update(Artifact artifact) throws Exception{
        String newVersion = remoteRepository.getLastVersion(artifact);
        return updateToVersion(artifact, newVersion);
    }

    /**
     * 更新到指定版本
     * @param artifact
     * @param version
     * @throws Exception
     */
    public ArtifactItem updateToVersion(Artifact artifact, String version) throws Exception{
        ArtifactItem artifactItem = getArtifactItem(artifact.getId(), version);
        if (null == artifactItem) {
            String taskName = artifactUpdateTasks.get(artifact.getId());
            if (null != taskName) {
                throw new Exception(String.format("仓库正在更新！执行步骤为【%s】", taskName));
            }
            try {
                artifactUpdateTasks.put(artifact.getId(), String.format("下载版本%s清单", version));
                if (Artifact.RepoType.DIFF.equals(Artifact.RepoType.valueOf(artifact.getRepoType()))) {
                    artifactItem = updateFromDiffRepo(artifact, version);
                } else {
                    artifactItem = updateFromMavenRepo(artifact, version);
                }
                artifactUpdateTasks.put(artifact.getId(), "记录日志");
                // update artifact info
                artifactItem = artifactItemRepository.save(artifactItem);
                artifact.setLast(artifactItem);
                artifact = artifactRepository.save(artifact);
            } finally {
                artifactUpdateTasks.remove(artifact.getId());
            }
        }
        return artifactItem;
    }

    private ArtifactItem updateFromDiffRepo(Artifact artifact, String version) throws Exception {
        // get last manifest
        byte[] data = remoteRepository.getManifestFile(artifact.getGroupId(), artifact.getArtifactId(), version);
        Manifest manifest = ObjectUtils.read(data);
        ArtifactItem artifactItem = new ArtifactItem(artifact, manifest);
        localRepository.saveManifestFile(artifact, artifactItem, data);

        artifactUpdateTasks.put(artifact.getId(), String.format("下载%s到%s差异包", artifact.getLastVersion(), version));
        // get diff
        data = remoteRepository.getDiffFile(artifact.getGroupId(), artifact.getArtifactId(), artifact.getLastVersion(), version);
        localRepository.saveDiffFile(artifact, artifactItem, data);
        return artifactItem;
    }

    private ArtifactItem updateFromMavenRepo(Artifact artifact, String version) throws Exception {
        MavenMetadata metadata = remoteRepository.getMetadata(artifact.getGroupId(), artifact.getArtifactId());
        ArtifactItem artifactItem = new ArtifactItem(artifact, metadata);
        artifactUpdateTasks.put(artifact.getId(), String.format("下载%s程序包", version));
        byte[] data = remoteRepository.getMavenFile(artifact.getGroupId(), artifact.getArtifactId(), version);
        localRepository.saveMavenFile(artifact, artifactItem, data);
        return artifactItem;
    }

    public Map<Long, String> getUpdateTasks() {
        return artifactUpdateTasks;
    }

    public String getUpdateTask(long artifactId) {
        return artifactUpdateTasks.get(artifactId);
    }

}
