package com.uas.erp.manage.client.service;

import com.uas.ops.diff.domain.Manifest;
import com.uas.erp.manage.client.entity.Artifact;
import com.uas.erp.manage.client.entity.ArtifactItem;
import com.uas.erp.manage.client.plugins.ProjectBuildPlugin;
import com.uas.erp.manage.client.util.*;
import com.uas.erp.manage.client.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.*;

import java.io.*;
import java.nio.charset.Charset;
import java.util.List;

/**
 * Created by Pro1 on 2017/6/17.
 */
@Service
public class LocalRepositoryService {

    @Autowired
    private SettingService settingService;

    private File getFile(String...dirOrFileNames) {
        return new File(StringUtils.arrayToDelimitedString(dirOrFileNames, File.separator));
    }

    private String getLocalRepositoryHome() {
        String localResourceHome = settingService.getValue("resource.path");
        Assert.notNull(localResourceHome);
        return localResourceHome + File.separator + "repository";
    }

    private String getLocalAttachHome() {
        String localResourceHome = settingService.getValue("resource.path");
        Assert.notNull(localResourceHome);
        return localResourceHome + File.separator + "attach";
    }

    private File getArtifactHome(Artifact artifact) {
        File dir = getFile(getLocalRepositoryHome(), artifact.getGroupId(), artifact.getArtifactId());
        if (!dir.exists()) {
            dir.mkdirs();
        }
        return dir;
    }

    private File getArtifactItemHome(Artifact artifact, ArtifactItem artifactItem) {
        File dir = getFile(getArtifactHome(artifact).getAbsolutePath(), artifactItem.getVersion());
        if (!dir.exists()) {
            dir.mkdirs();
        }
        return dir;
    }

    public void saveManifestFile(Artifact artifact, ArtifactItem artifactItem, byte[] data) throws IOException{
        File dir = getArtifactItemHome(artifact, artifactItem);
        File manifestFile = new File(dir, ".manifest");
        FileCopyUtils.copy(data, manifestFile);
    }

    public void saveDiffFile(Artifact artifact, ArtifactItem artifactItem, byte[] data) throws IOException{
        File dir = getArtifactItemHome(artifact, artifactItem);
        ZipUtils.unzip(data, dir.getAbsolutePath());
    }

    public void saveMavenFile(Artifact artifact, ArtifactItem artifactItem, byte[] data) throws IOException{
        File dir = getArtifactItemHome(artifact, artifactItem);
        ZipUtils.unzip(data, dir.getAbsolutePath());
    }

    public File getManifestFile(Artifact artifact, ArtifactItem artifactItem) {
        File dir = getArtifactItemHome(artifact, artifactItem);
        return new File(dir, ".manifest");
    }

    public Manifest getManifest(Artifact artifact, ArtifactItem artifactItem) throws Exception{
        File manifestFile = getManifestFile(artifact, artifactItem);
        if (manifestFile.exists()) {
            Manifest manifest = ObjectUtils.read(FileCopyUtils.copyToByteArray(manifestFile));
            return manifest;
        }
        return null;
    }

    /**
     * 按指定版本清单收集差异文件进行编译
     * @param buildName
     * @param plugins
     * @throws Exception
     */
    public File buildByManifest(Artifact artifact, List<ArtifactItem> allItems, ArtifactItem currentItem, String buildName, ProjectBuildPlugin...plugins) throws Exception{
        Manifest manifest = getManifest(artifact, currentItem);
        if (null != manifest) {
            File tmpdir = FileUtils.createTemporaryDirectory();
            File buildFile = getBuildFile(artifact, buildName);
            try {
                copyResourcesByManifest(artifact, allItems, manifest, tmpdir.getAbsolutePath());
                // plugins after copy-resources
                if (null != plugins) {
                    for (ProjectBuildPlugin plugin:plugins) {
                        plugin.onBuild(tmpdir);
                    }
                }
                ZipUtils.zip(tmpdir, buildFile);
                return buildFile;
            } finally {
                FileSystemUtils.deleteRecursively(tmpdir);
            }
        }
        return null;
    }

    /**
     * 编译后文件默认存放在{repository}/{groupId}/{artifactId}下面
     * @param artifact
     * @param buildName
     * @return
     */
    private File getBuildFile(Artifact artifact, String buildName) {
        // tomcat的命名方法
        if (StringUtils.isEmpty(buildName) || "/".equals(buildName)) {
            buildName = "ROOT";
        }
        return getFile(getArtifactHome(artifact).getAbsolutePath(), buildName + "." + artifact.getPackaging());
    }

    /**
     * 按清单搜集文件
     * @param artifact
     * @param manifest
     * @param target
     */
    private void copyResourcesByManifest(Artifact artifact, List<ArtifactItem> allItems, Manifest manifest, String target) throws IOException{
        if (null != manifest.getChildren()) {
            if (!"/".equals(manifest.getPath())) {
                new File(target + manifest.getPath()).mkdirs();
            }
            for (Manifest child:manifest.getChildren()) {
                copyResourcesByManifest(artifact, allItems, child, target);
            }
        } else {
            File file = getResourceByVersion(artifact, allItems, manifest.getPath(), manifest.getVersion());
            if (null != file) {
                FileCopyUtils.copy(file, new File(target + manifest.getPath()));
            }
        }
    }

    /**
     * 按版本从高到低依次遍历查找资源
     * @param artifact
     * @param filePath
     * @param needVersion
     * @return
     */
    private File getResourceByVersion(Artifact artifact, List<ArtifactItem> allItems, String filePath, String needVersion) {
        for (ArtifactItem item:allItems) {
            // less or equals {version}
            if (item.getVersion().compareTo(needVersion) <= 0) {
                File file = getFile(getArtifactItemHome(artifact, item).getAbsolutePath(), filePath);
                if (file.exists()) {
                    return file;
                }
            }
        }
        return null;
    }

    /**
     * 字符串保存到本地附件
     * @param resourceStr
     * @return
     * @throws IOException
     */
    public String saveResourceAsAttach(String resourceStr) throws IOException{
        File file = new File(getLocalAttachHome(), StringUtil.uuid());
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        FileCopyUtils.copy(resourceStr.getBytes(Charset.forName("UTF-8")), file);
        return file.getName();
    }

    public File getAttach(String attachName){
        return new File(getLocalAttachHome(), attachName);
    }

    /**
     * 读取本地附件文件
     * @param attachName
     * @return
     * @throws IOException
     */
    public byte[] getAttachAsByteArray(String attachName) throws IOException{
        File file = getAttach(attachName);
        if (file.exists()) {
            return FileCopyUtils.copyToByteArray(file);
        }
        return null;
    }

    /**
     * 读取本地附件文件字符串内容
     * @param attachName
     * @return
     * @throws IOException
     */
    public String getAttachAsString(String attachName) throws IOException{
        File file = getAttach(attachName);
        if (file.exists()) {
            FileReader reader = new FileReader(file);
            try {
                return FileCopyUtils.copyToString(reader);
            } finally {
                IOUtils.closeQuietly(reader);
            }
        }
        return null;
    }

    /**
     * 复制maven文件编译
     * @param artifact
     * @param currentItem
     * @param buildName
     * @param plugins
     * @return
     * @throws Exception
     */
    public File buildMavenFile(Artifact artifact, ArtifactItem currentItem, String buildName, ProjectBuildPlugin...plugins) throws Exception{
        File tmpdir = FileUtils.createTemporaryDirectory();
        File buildFile = getBuildFile(artifact, buildName);
        try {
            FileSystemUtils.copyRecursively(getArtifactItemHome(artifact, currentItem), tmpdir);
            // plugins after copy-resources
            if (null != plugins) {
                for (ProjectBuildPlugin plugin:plugins) {
                    plugin.onBuild(tmpdir);
                }
            }
            ZipUtils.zip(tmpdir, buildFile);
            return buildFile;
        } finally {
            FileSystemUtils.deleteRecursively(tmpdir);
        }
    }

}
