package com.uas.erp.manage.client.service;

import com.thoughtworks.xstream.XStream;
import com.uas.erp.manage.client.domain.MavenMetadata;
import com.uas.erp.manage.client.domain.MavenPom;
import com.uas.erp.manage.client.entity.Artifact;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.net.URL;

/**
 * Created by Pro1 on 2017/6/17.
 */
@Service
public class RemoteRepositoryService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private SettingService settingService;

    private String getDiffRepositoryUrl() {
        return settingService.getValue("diff.repo.url");
    }

    private String getMavenRepositoryUrl() {
        return settingService.getValue("maven.repo.url");
    }

    /**
     * 获取最新版本
     * @param artifact
     * @return
     */
    public String getLastVersion(Artifact artifact) throws Exception{
        if (Artifact.RepoType.DIFF.equals(Artifact.RepoType.valueOf(artifact.getRepoType()))) {
            return restTemplate.getForObject(getDiffRepositoryUrl() + "/version?groupId={groupId}&artifactId={artifactId}", String.class,
                    artifact.getGroupId(), artifact.getArtifactId());
        }
        MavenMetadata metadata = getMetadata(artifact.getGroupId(), artifact.getArtifactId());
        return null == metadata ? null : metadata.getVersioning().getLatest();
    }

    /**
     * (diff)下载版本文件清单
     * @param groupId
     * @param artifactId
     * @param version
     * @return
     */
    public byte[] getManifestFile(String groupId, String artifactId, String version) {
        return restTemplate.getForObject(getDiffRepositoryUrl() + "/manifest?groupId={groupId}&artifactId={artifactId}&version={version}",
                byte[].class, groupId, artifactId, version);
    }

    /**
     * (diff)下载版本间差异文件包
     * @param groupId
     * @param artifactId
     * @param fromVersion
     * @param toVersion
     * @return
     */
    public byte[] getDiffFile(String groupId, String artifactId, String fromVersion, String toVersion) {
        return restTemplate.getForObject(getDiffRepositoryUrl() + "/diff?groupId={groupId}&artifactId={artifactId}&fromVersion={fromVersion}&toVersion={toVersion}",
                byte[].class, groupId, artifactId, fromVersion, toVersion);
    }

    /**
     * (maven)版本信息
     * @param groupId
     * @param artifactId
     * @return
     */
    public MavenMetadata getMetadata(String groupId, String artifactId) throws Exception{
        XStream xStream = new XStream();
        xStream.setMode(XStream.NO_REFERENCES);
        xStream.processAnnotations(new Class[]{MavenMetadata.class, MavenMetadata.Versioning.class});
        return (MavenMetadata) xStream.fromXML(new URL(getMavenRepositoryUrl() + "/" + groupId.replace(".", "/") + artifactId + "/maven-metadata.xml"));
    }

    /**
     * (maven)POM文件
     * @param groupId
     * @param artifactId
     * @param version
     * @return
     * @throws Exception
     */
    public MavenPom getPom(String groupId, String artifactId, String version) throws Exception{
        XStream xStream = new XStream();
        xStream.setMode(XStream.NO_REFERENCES);
        xStream.processAnnotations(MavenPom.class);
        return (MavenPom) xStream.fromXML(new URL(String.format(getMavenRepositoryUrl() + "/%s/%s/%s/%s-%s.pom",
                groupId.replace(".", "/"), artifactId, version, artifactId, version)));
    }

    /**
     * (maven)仓库文件
     * @param groupId
     * @param artifactId
     * @param version
     * @return
     * @throws Exception
     */
    public byte[] getMavenFile(String groupId, String artifactId, String version) throws Exception {
        MavenPom pom = getPom(groupId, artifactId, version);
        return restTemplate.getForObject(getMavenRepositoryUrl() + "/{groupPath}/{artifactId}/{version}/{artifactId}-{version}.{packaging}",
                byte[].class, groupId.replace(".", "/"), artifactId, version, artifactId, version, pom.getPackaging());
    }

}
